/*
  This is a part of the TDRD Source Code Samples. 
  Copyright (C) 1997-2001 TDRD Ltd.,Part.
  All rights reserved. 

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright 1997-2001 TDRD Ltd.,Part. All Rights Reserved.

  PROGRAM: sample.c

  PURPOSE: An example application for pgsnd.dll

  PLATFORMS: Windows CE
*/

#include "sample.h"

#define MAX_LOADSTRING 100

HINSTANCE   hInst;
HWND        hWndMain;

static ATOM MyRegisterClass(HINSTANCE, LPTSTR);
static BOOL InitInstance(HINSTANCE, int);

HWND       hPlayBtn[4];
HWND       hFileOpen[4];
HWND       hFileName[4];
TCHAR      szFileName[128];

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
  MSG msg;

  if (!InitInstance (hInstance, nCmdShow)) 
    return FALSE;

  while (GetMessage(&msg, NULL, 0, 0)) 
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  /* un-hook waveform out-put device and free all allocated mamory */
  pg_WaveClose();
  return msg.wParam;
}

static ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
  WNDCLASS  wc;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc  = (WNDPROC) WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_WAVEMIXER));
  wc.hCursor = 0;
  wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
  wc.lpszMenuName = 0;
  wc.lpszClassName = szWindowClass;

  return RegisterClass(&wc);
}

static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
  HWND  hWnd;
  TCHAR  szTitle[MAX_LOADSTRING];
  TCHAR  szWindowClass[MAX_LOADSTRING];

  hInst = hInstance;
  LoadString(hInstance, IDC_WAVEMIXER, szWindowClass, MAX_LOADSTRING);
  MyRegisterClass(hInstance, szWindowClass);
  LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
  hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

  if (!hWnd)
    return FALSE;

  /* hook waveform out-put device and startup wave mixer */
  pg_WaveInit(hWnd,4,44100,8192);
  hWndMain = hWnd;
  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  return TRUE;
}

static void CreateAppMenu(HWND hWnd)
{
  //Create the menubar.
  SHMENUBARINFO mbi;

  memset (&mbi, 0, sizeof (SHMENUBARINFO));
  mbi.cbSize     = sizeof (SHMENUBARINFO);
  mbi.hwndParent = hWnd;
  mbi.nToolBarId = IDR_MENUBAR;
  mbi.hInstRes   = hInst;
  mbi.nBmpId     = 0;
  mbi.cBmpImages = 0;    

  SHCreateMenuBar(&mbi);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  int wmId, wmEvent, y, n;

  switch (message) 
  {
  case WM_COMMAND:
    wmId    = LOWORD(wParam); 
    wmEvent = HIWORD(wParam); 

    switch (wmId)
    {
    case 5000:
    case 5001:
    case 5002:
    case 5003:
    case 5004:
      lstrcpy(szFileName,TEXT("wav"));

      if(DialogBoxParam(hInst, MAKEINTRESOURCE(300), hWnd, (DLGPROC)OpenDlg, (long)szFileName))
      {
        if(pg_LoadWaveFile(szFileName,wmId-5000))
          SetWindowText(hFileName[wmId-5000],L"<not PCM format>");
        else
        {
          // strip out file path to display
          TCHAR *lp1 = szFileName + lstrlen(szFileName);

          while(lp1 > szFileName && !(*lp1 == '\\' || *lp1 ==':'))
            lp1--;

          if(*lp1 == '\\' || *lp1 == ':')
            lp1++;

          SetWindowText(hFileName[wmId-5000],lp1);
        }
      }
      break;

    case 4000:
    case 4001:
    case 4002:
    case 4003:
      pg_WavePlay(wmId-4000,FALSE);
      break;

    case 4004:
    case 4005:
    case 4006:
    case 4007:
      MessageBox(hWnd,TEXT("Not available for demo verison"),TEXT("PGSND"),MB_OK);
      break;

    case 4008:
      pg_PlayMod();
      break;

    case ID_FILE_ABOUT:
      DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
      break;

    case ID_FILE_EXIT:
      //DestroyWindow(hWnd);
      break;

    default:
      return DefWindowProc(hWnd, message, wParam, lParam);
    }
    break;

  case WM_ACTIVATE:
    pg_WaveActivate((BOOL)wParam);
    break;

  case WM_CREATE:
    CreateAppMenu(hWnd);

    for(n=0; n<4; n++)
    {
      y = 30;
      hFileName[n] = CreateWindow(L"EDIT",L"",WS_VISIBLE|WS_CHILD|WS_BORDER|ES_LEFT,10,y+n*20,120,18,hWnd,(HMENU)(3000+n),hInst,0);
      hPlayBtn[n] = CreateWindow(L"BUTTON",L"Play",WS_VISIBLE|WS_CHILD|BS_PUSHBUTTON,140,y+n*20,50,18,hWnd,(HMENU)(4000+n),hInst,0);
      hFileOpen[n] = CreateWindow(L"BUTTON",L"...",WS_VISIBLE|WS_CHILD|BS_PUSHBUTTON,200,y+n*20,30,18,hWnd,(HMENU)(5000+n),hInst,0);
    }

    SetTimer(hWnd,1,100,0);
    break;

  case WM_DESTROY:
    PostQuitMessage(0);
    break;

  default:
    return DefWindowProc(hWnd, message, wParam, lParam);
  }

  return 0;
}

LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
  case WM_COMMAND:
    if ((LOWORD(wParam) == IDOK) || (LOWORD(wParam) == IDCANCEL))
    {
      EndDialog(hDlg, LOWORD(wParam));
      return TRUE;
    }
    break;
  }

  return FALSE;
}

