/*#############################################################################
# TCONVERT.H
#
# SCA Software International S.A.
# http://www.scasoftware.com
# scaadmin@scasoftware.com
#
# Copyright (c) 2000 SCA Software International S.A.
#
# Date: 01.05.2000
# Author: Zoran M.Todorovic
#
# This software is provided "AS IS", without a warranty of any kind.
# You are free to use/modify this code but leave this header intact.
#
#############################################################################*/
#ifndef __TCONVERT_H__
#define __TCONVERT_H__

#ifndef _INC_TCHAR
# include <tchar.h>
#endif
#ifndef _INC_CRTDBG
#ifdef ORB_WINDOWS_32
# include <crtdbg.h>
#else
#define _ASSERT ASSERT
#endif

#endif
#ifndef _WINDOWS_
# include <windows.h>
#endif

//=============================================================================
// class _tochar
// This class converts either WCHAR or CHAR string to a new CHAR string.
// Memory is allocated/deallocated using new/delete
//=============================================================================

class _tochar {
private:
  BOOL m_bAutoDelete;
  LPSTR m_szBuffer;

public:
  _tochar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText)+1;
    m_szBuffer = new CHAR [nLen];
    wcstombs(m_szBuffer, wszText, nLen);
  }
  _tochar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_szBuffer = new CHAR [nLen];
    strcpy(m_szBuffer, szText);
  }
  ~_tochar()
  {
    if (m_bAutoDelete) {
      _ASSERT(m_szBuffer);
      delete [] m_szBuffer;
    }
  }
  operator LPSTR()
  {
    _ASSERT(m_szBuffer);
    return (LPSTR)m_szBuffer;
  }
  operator LPCSTR()
  {
    _ASSERT(m_szBuffer);
    return (LPCSTR)m_szBuffer;
  }
};

//=============================================================================
// class _towchar
// This class converts either WCHAR or CHAR string to a new WCHAR string.
// Memory is allocated/deallocated using new/delete
//=============================================================================

class _towchar {
private:
  BOOL m_bAutoDelete;
  LPWSTR m_wszBuffer;

public:
  _towchar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText)+1;
    m_wszBuffer = new WCHAR [nLen];
    wcscpy(m_wszBuffer, wszText);
  }
  _towchar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_wszBuffer = new WCHAR [nLen];
    mbstowcs(m_wszBuffer, szText, nLen);
  }
  ~_towchar()
  {
    if (m_bAutoDelete) {
      _ASSERT(m_wszBuffer);
      delete [] m_wszBuffer;
    }
  }
  operator LPWSTR()
  {
    _ASSERT(m_wszBuffer);
    return (LPWSTR)m_wszBuffer;
  }
  operator LPCWSTR()
  {
    _ASSERT(m_wszBuffer);
    return (LPCWSTR)m_wszBuffer;
  }
};

//=============================================================================
// class _totchar
// This class converts a TCHAR string to a new TCHAR string.
// Memory is allocated/deallocated using new/delete
//=============================================================================

class _totchar {
private:
  BOOL m_bAutoDelete;
  LPTSTR m_tszBuffer;

public:
  _totchar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_tszBuffer = new TCHAR [nLen];
    #if defined(UNICODE) || defined(_UNICODE)
    mbstowcs(m_tszBuffer, szText, nLen);
    #else
    strcpy(m_tszBuffer, szText);
    #endif
  }
  _totchar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText) + 1;
    m_tszBuffer = new TCHAR [nLen];
    #if defined(UNICODE) || defined(_UNICODE)
    wcscpy(m_tszBuffer, wszText);
    #else
    wcstombs(m_tszBuffer, wszText, nLen);
    #endif
  }
  ~_totchar()
  {
    if (m_bAutoDelete) {
      _ASSERT(m_tszBuffer);
      delete [] m_tszBuffer;
    }
  }
  operator LPTSTR()
  {
    _ASSERT(m_tszBuffer);
    return (LPTSTR) m_tszBuffer;
  }
  operator LPCTSTR()
  {
    _ASSERT(m_tszBuffer);
    return (LPCTSTR) m_tszBuffer;
  }
};

#if 0
//=============================================================================
// class _cochar
// This class converts either WCHAR or CHAR string to a new CHAR string.
// Memory is allocated/deallocated using CoTaskMemAlloc/CoTaskMemFree.
//=============================================================================

class _cochar {
private:
  BOOL m_bAutoDelete;
  LPSTR m_szBuffer;

public:
  _cochar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText)+1;
    m_szBuffer = (LPSTR)::CoTaskMemAlloc(nLen * sizeof(CHAR));
    wcstombs(m_szBuffer, wszText, nLen);
  }
  _cochar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_szBuffer = (LPSTR)::CoTaskMemAlloc(nLen * sizeof(CHAR));
    strcpy(m_szBuffer, szText);
  }
  ~_cochar()
  {
    if (m_bAutoDelete)
      ::CoTaskMemFree(m_szBuffer);
  }
  operator LPSTR()
  {
    return (LPSTR)m_szBuffer;
  }
  operator LPCSTR()
  {
    return (LPCSTR)m_szBuffer;
  }
};

//=============================================================================
// class _towchar
// This class converts either WCHAR or CHAR string to a new WCHAR string.
// Memory is allocated/deallocated using CoTaskMemAlloc/CoTaskMemFree
//=============================================================================

class _cowchar {
private:
  BOOL m_bAutoDelete;
  LPWSTR m_wszBuffer;

public:
  _cowchar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText)+1;
    m_wszBuffer = (LPWSTR)::CoTaskMemAlloc(nLen * sizeof(WCHAR));
    wcscpy(m_wszBuffer, wszText);
  }
  _cowchar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_wszBuffer = (LPWSTR)::CoTaskMemAlloc(nLen * sizeof (WCHAR));
    mbstowcs(m_wszBuffer, szText, nLen);
  }
  ~_cowchar()
  {
    if (m_bAutoDelete)
      ::CoTaskMemFree(m_wszBuffer);
  }
  operator LPWSTR()
  {
    return (LPWSTR)m_wszBuffer;
  }
  operator LPCWSTR()
  {
    return (LPCWSTR)m_wszBuffer;
  }
};

//=============================================================================
// class _cotchar
// This class converts a TCHAR string to a new TCHAR string.
// Memory is allocated/deallocated using CoTaskMemAlloc/CoTaskMemFree
//=============================================================================

class _cotchar {
private:
  BOOL m_bAutoDelete;
  LPTSTR m_tszBuffer;

public:
  _cotchar(LPCSTR szText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(szText);
    int nLen = strlen(szText) + 1;
    m_tszBuffer = (LPTSTR)::CoTaskMemAlloc(nLen * sizeof(TCHAR));
    #if defined(UNICODE) || defined(_UNICODE)
    mbstowcs(m_tszBuffer, szText, nLen);
    #else
    strcpy(m_tszBuffer, szText);
    #endif
  }
  _cotchar(LPCWSTR wszText, BOOL bAutoDelete = TRUE)
  {
    m_bAutoDelete = bAutoDelete;
    _ASSERT(wszText);
    int nLen = wcslen(wszText) + 1;
    m_tszBuffer = (LPTSTR)::CoTaskMemAlloc(nLen * sizeof(TCHAR));
    #if defined(UNICODE) || defined(_UNICODE)
    wcscpy(m_tszBuffer, wszText);
    #else
    wcstombs(m_tszBuffer, wszText, nLen);
    #endif
  }
  ~_cotchar()
  {
    if (m_bAutoDelete)
      ::CoTaskMemFree(m_tszBuffer);
  }
  operator LPTSTR()
  {
    return (LPTSTR) m_tszBuffer;
  }
  operator LPCTSTR()
  {
    return (LPCTSTR) m_tszBuffer;
  }
};
#endif
#endif


/*#############################################################################
# End of file
#############################################################################*/

#if !defined(AFX_UNICODESTRING_H__1C328430_CFAB_11D2_A733_0000E83A16D9__INCLUDED_)
#define AFX_UNICODESTRING_H__1C328430_CFAB_11D2_A733_0000E83A16D9__INCLUDED_

bool IsValidString(const char * lpsz, int nLength = -1);
bool IsValidString(const wchar_t * lpsz, int nLength = -1);


#ifdef NOSUPPORT_COM
#define _NO_BSTR_SUPPORT
#endif

//#if UNDER_CE==201
//#define _NO_BSTR_SUPPORT
//#endif
#define _NO_BSTR_SUPPORT

#define CUnicodeString ustring
class ustring  
{
protected:

	struct Srep 
	{
		wchar_t *s;	
		int sz;
		int n;

		Srep(int nsz, const wchar_t* p)
		{
			n = 1;
			sz = nsz;
			s = new wchar_t[sz + 1]; // add space for the terminator
			wcscpy(s,p);
		}

		~Srep() { delete[] s; }

		Srep * get_own_copy() // clone if neccessary
		{
			if (n==1) return this;
			n--;
			return new Srep(sz,s);
		}

		void assign(int nsz, const wchar_t* p)
		{
			if (sz!=nsz)
			{
				delete[] s;
				sz = nsz;
				s = new wchar_t[sz+1];
			}
			wcscpy(s,p);
		}

	private:	// prevent copying:
		Srep(const Srep&); 
		Srep& operator=(const Srep&);

	};
	
	Srep * rep;
	class	Cref;
	
public:

	class ustring::Cref // reference to s[i]
	{
		friend class ustring;
		ustring &s;
		int i;
		Cref(ustring &ss, int ii) : s(ss), i(ii) { }

	public:
		operator wchar_t() { return s.read(i); }	// yield value
		void operator=(wchar_t c) { s.write(i,c); }	// change value

	};

#ifndef _NO_BSTR_SUPPORT
	//BSTR SetSysString(BSTR* pbstr) const;
	BSTR AllocSysString() const;
#endif
	
	int FindOneOf(const char * lpszCharSet) const;
	int FindOneOf(const wchar_t * lpszCharSet) const;
	
	int ReverseFind(char ch) const;
	int ReverseFind(wchar_t ch) const;

	int Find(char ch, int nStart = 0) const;
	int Find(wchar_t ch, int nStart = 0) const;
	int Find(const char * lpszSub, int nStart = 0) const;
	int Find(const wchar_t * lpszSub, int nStart = 0) const;
	
	void TrimRight();
	void TrimLeft();

//	int Delete(int nIndex, int nCount = 1);
	int Insert(int nIndex, const char * pstr);
	int Insert(int nIndex, const wchar_t * pstr);
	int Insert(int nIndex, char ch);
	int Insert(int nIndex, wchar_t ch);
//	int Remove(char ch);
//	int Remove(wchar_t ch);
	
	void MakeReverse();
	void MakeLower();
	void MakeUpper();

	int	CompareNoCase(const char * lpsz);
	int CompareNoCase(const wchar_t * lpsz);
	
	int Compare(const char * lpsz);
	int Compare(const wchar_t * lpsz);
	
	void SetAt(int dwPos, char ch);
	void SetAt(int dwPos, wchar_t ch);
	
	void Empty();
	bool IsEmpty() const;

	wchar_t * GetBuffer();
	operator const wchar_t * () const;

	ustring Mid(int dwStart, int dwCount);
	ustring Right(int dwCount);
	ustring Left(int dwCount);
	
	void Format(const wchar_t * pstrFormat, ...);
	void Format(const char * pstrFormat, ...);
	int GetLength();

	ustring();
	ustring(const wchar_t * pstrString);
	ustring(const char * pstrString);
	ustring(const ustring& strString);


	const ustring& operator =(const ustring& strString);
	const ustring& operator =(const wchar_t * pstrString);
	const ustring& operator =(const char * pstrString);
	const ustring& operator =(wchar_t ch);
	const ustring& operator =(char ch);

	void operator += (const ustring& usString);
	void operator += (const wchar_t * pstrString);
	void operator += (const char * pstrString);
	void operator += (wchar_t wch);
	void operator += (char ch);
	

	friend ustring operator+(const ustring& s1, const ustring& s2);
	friend ustring operator+(const ustring& s, const char * pstrString);
	friend ustring operator+(const char * pstrString, const ustring& s);
	friend ustring operator+(const ustring& s, const char ch);
	friend ustring operator+(const char ch, const ustring& s);
	friend ustring operator+(const ustring& s, const wchar_t * pstrString);
	friend ustring operator+(const wchar_t * pstrString, const ustring& s);
	friend ustring operator+(const ustring& s, const wchar_t ch);
	friend ustring operator+(const wchar_t ch, const ustring& s);
	
	wchar_t GetAt(int dwPos) const;
	virtual ~ustring();
	
	int		size() const { return rep->sz; }
	wchar_t operator[](int i) const 
	{ 
		if (check(i)) return rep->s[i];  _ASSERT(FALSE); 
		return 0;
	}
	Cref    operator[](int i)  { if (check(i)) return Cref(*this,i);   
							    _ASSERT(FALSE); return Cref(*this,0); }
	bool	check(int i) const { if ( i<0 || rep->sz<=i) { return false; } return true; }
	wchar_t read(int i) const { return rep->s[i];}
	void	write(int i, wchar_t c) { rep=rep->get_own_copy(); rep->s[i] = c;}
	
protected:
	friend		struct CUnicodeString_less;
	friend		void PrintCUnicodeString(const ustring & s);
	friend		bool operator < (const ustring& p, const ustring& q);
	friend		bool operator ==(const ustring& s1, const ustring& s2);
	friend		bool operator ==(const ustring& s1, const wchar_t * s2);
	friend		bool operator ==(const ustring& s1, const char * s2);
	friend		bool operator ==(const wchar_t * s1, const ustring& s2);
	friend		bool operator ==(const char * s1, const ustring& s2);
	friend		bool operator !=(const ustring& s1, const ustring& s2);
	friend		bool operator !=(const ustring& s1, const wchar_t * s2);
	friend		bool operator !=(const ustring& s1, const char * s2);
	friend		bool operator !=(const wchar_t * s1, const ustring& s2);
	friend		bool operator !=(const char * s1, const ustring& s2);


};

bool operator ==(const ustring& s1, const ustring& s2);
bool operator ==(const ustring& s1, const wchar_t * s2);
bool operator ==(const ustring& s1, const char * s2);
bool operator ==(const wchar_t * s1, const ustring& s2);
bool operator ==(const char * s1, const ustring& s2);
bool operator !=(const ustring& s1, const ustring& s2);
bool operator !=(const ustring& s1, const wchar_t * s2);
bool operator !=(const ustring& s1, const char * s2);
bool operator !=(const wchar_t * s1, const ustring& s2);
bool operator !=(const char * s1, const ustring& s2);

// For STL
bool operator < (const ustring& s1, const ustring& s2);

int _customcmp(const wchar_t * s1, const wchar_t * s2);
int _customcmp(const char * s1, const char *  s2);

// For STL containers
struct CUnicodeString_less
{
	bool operator() ( const ustring & p, const ustring & q ) const
	{ 
		return _customcmp(p.rep->s,p.rep->s) < 0; 
	}
};

#define USTRING2ASCII(USTR, AUTOPTR_CHAR) \
	auto_ptr<char> AUTOPTR_CHAR(new char[USTR.GetLength()+1]); \
	pc_wtoa((const wchar_t*)USTR, AUTOPTR_CHAR.get() , USTR.GetLength() );  \
	((char*)(AUTOPTR_CHAR.get()))[USTR.GetLength()] = '\0';


void PrintCUnicodeString(const ustring & s);

#endif // !defined(AFX_UNICODESTRING_H__1C328430_CFAB_11D2_A733_0000E83A16D9__INCLUDED_)

